<?php

namespace Litecms\Coupon\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Coupon\Http\Requests\CouponsRequest;
use Litecms\Coupon\Interfaces\CouponsRepositoryInterface;
use Litecms\Coupon\Models\Coupons;

/**
 * Resource controller class for coupons.
 */
class CouponsResourceController extends BaseController
{

    /**
     * Initialize coupons resource controller.
     *
     * @param type CouponsRepositoryInterface $coupons
     *
     * @return null
     */
    public function __construct(CouponsRepositoryInterface $coupons)
    {
        parent::__construct();
        $this->repository = $coupons;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Coupon\Repositories\Criteria\CouponsResourceCriteria::class);
    }

    /**
     * Display a list of coupons.
     *
     * @return Response
     */
    public function index(CouponsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Coupon\Repositories\Presenter\CouponsPresenter::class)
                ->$function();
        }

        $coupons = $this->repository->paginate();

        return $this->response->title(trans('coupon::coupons.names'))
            ->view('coupon::coupons.index', true)
            ->data(compact('coupons'))
            ->output();
    }

    /**
     * Display coupons.
     *
     * @param Request $request
     * @param Model   $coupons
     *
     * @return Response
     */
    public function show(CouponsRequest $request, Coupons $coupons)
    {

        if ($coupons->exists) {
            $view = 'coupon::coupons.show';
        } else {
            $view = 'coupon::coupons.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('coupon::coupons.name'))
            ->data(compact('coupons'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new coupons.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CouponsRequest $request)
    {

        $coupons = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('coupon::coupons.name')) 
            ->view('coupon::coupons.create', true) 
            ->data(compact('coupons'))
            ->output();
    }

    /**
     * Create new coupons.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CouponsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $coupons                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('coupon::coupons.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('coupon/coupons/' . $coupons->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/coupon/coupons'))
                ->redirect();
        }

    }

    /**
     * Show coupons for editing.
     *
     * @param Request $request
     * @param Model   $coupons
     *
     * @return Response
     */
    public function edit(CouponsRequest $request, Coupons $coupons)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('coupon::coupons.name'))
            ->view('coupon::coupons.edit', true)
            ->data(compact('coupons'))
            ->output();
    }

    /**
     * Update the coupons.
     *
     * @param Request $request
     * @param Model   $coupons
     *
     * @return Response
     */
    public function update(CouponsRequest $request, Coupons $coupons)
    {
        try {
            $attributes = $request->all();

            $coupons->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('coupon::coupons.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('coupon/coupons/' . $coupons->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('coupon/coupons/' . $coupons->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the coupons.
     *
     * @param Model   $coupons
     *
     * @return Response
     */
    public function destroy(CouponsRequest $request, Coupons $coupons)
    {
        try {

            $coupons->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('coupon::coupons.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('coupon/coupons/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('coupon/coupons/' . $coupons->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple coupons.
     *
     * @param Model   $coupons
     *
     * @return Response
     */
    public function delete(CouponsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('coupon::coupons.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('coupon/coupons'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/coupon/coupons'))
                ->redirect();
        }

    }

    /**
     * Restore deleted coupons.
     *
     * @param Model   $coupons
     *
     * @return Response
     */
    public function restore(CouponsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('coupon::coupons.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/coupon/coupons'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/coupon/coupons/'))
                ->redirect();
        }

    }

}
